#version 120

#define BLOOM_STRENGTH 0.5 //[0.0 0.2 0.4 0.5 0.6 0.8 1.0]
#define GAMMA_ADJUSTMENT 0.2 //[0.0 0.1 0.2 0.3 0.4 0.5]
#define BLOOM_OFF 0
#define BLOOM_ON 1
#define BLOOM_TOGGLE BLOOM_ON //[BLOOM_OFF BLOOM_ON]

uniform sampler2D colortex0;
uniform float viewWidth;
uniform float viewHeight;

varying vec2 texcoord;

// Bloom function
void Bloom(inout vec3 color) {
    #if BLOOM_TOGGLE == BLOOM_ON
        vec3 blur = vec3(0.0);
        float totalWeight = 0.0;
        for (int x = -4; x <= 4; x++) {
            for (int y = -4; y <= 4; y++) {
                vec2 offset = vec2(float(x) / viewWidth, float(y) / viewHeight);
                float weight = 1.0 / (1.0 + float(x * x + y * y));
                blur += texture2D(colortex0, texcoord + offset).rgb * weight;
                totalWeight += weight;
            }
        }
        blur /= totalWeight;
        color = mix(color, blur, BLOOM_STRENGTH);
    #endif
}

void AdjustGamma(inout vec3 color) {
    color = pow(color, vec3(1.0 / (1.0 + GAMMA_ADJUSTMENT)));
}

void main() {
    vec3 color = texture2D(colortex0, texcoord).rgb;
    Bloom(color);
    AdjustGamma(color);
    gl_FragColor = vec4(color, 1.0);
}
